#Requires -Version 5.1
#Requires -Modules @{ ModuleName="Az.Accounts"; RequiredVersion="2.13.0" }
#Requires -Modules @{ ModuleName="Az.Resources"; RequiredVersion="6.10.0" }
#Requires -Modules @{ ModuleName="Az.Storage"; RequiredVersion="5.10.0" }
#Requires -Modules @{ ModuleName="Az.KeyVault"; RequiredVersion="4.11.0" }

Param(
    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $TenantId,

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $SubscriptionId,

    [Parameter(Mandatory = $false)] 
    [ValidateNotNull()]
    [string]
    $ResourceGroupName = 'KelverionAutomationPortal',

    [Parameter(Mandatory = $false)] 
    [ValidateNotNull()]
    [System.Management.Automation.Credential()]
    [System.Management.Automation.PSCredential] 
    $SqlAdminCredential = [System.Management.Automation.PSCredential]::Empty,

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $Location,

    [Parameter(Mandatory = $false)]
    [ValidateNotNull()]
    [bool] 
    $UseCurrentAzContext = $false,

    [Parameter(Mandatory = $false)] 
    [ValidateSet("AzureCloud", "AzureUsGovernment", "AzureGermanCloud", "AzureChinaCloud")]
    [string]
    $Environment = "AzureCloud"
)

$ErrorActionPreference = 'Stop'

$OptionalParameters = New-Object -TypeName Hashtable
$TemplateArgs = New-Object -TypeName Hashtable

#####################################################################
# Connect to Azure
#####################################################################
Write-Progress -Activity "Deploying App Service" -Id 2 -Status "Checking Az context" -PercentComplete 16
$AzContext = Get-AzContext
if($UseCurrentAzContext -eq $false -or $null -eq $AzContext -or $AzContext.Tenant.Id -ne $TenantId -or $AzContext.Subscription.Id -ne $SubscriptionId) {
    Connect-AzAccount -TenantId $TenantId -SubscriptionId $SubscriptionId -Environment $Environment 
}

#####################################################################
# SQL Admin Credential
#####################################################################
if($SqlAdminCredential -eq [System.Management.Automation.PSCredential]::Empty) {
    $SqlAdminCredential = Get-Credential -Message "Set the Automation Portal SQL Server/Database administrator login and password."
}

#####################################################################
# Validate
#####################################################################
Write-Progress -Activity "Deploying App Service" -Id 2 -Status "Checking resource group name" -PercentComplete 32
if ($null -ne (Get-AzResourceGroup -Name $ResourceGroupName -Verbose -ErrorAction SilentlyContinue)) {
    Write-Host -f Red "The Resource Group: $ResourceGroupName already exists."
    Exit 1
}

#####################################################################
# Deploy
#####################################################################
Write-Progress -Activity "Deploying App Service" -Id 2 -Status "Deploying" -PercentComplete 48
$ArtifactsLocationName = '_artifactsLocation'
$ArtifactsLocationSasTokenName = '_artifactsLocationSasToken'

$OptionalParameters["sqlAdministratorLogin"] = $SqlAdminCredential.UserName
$OptionalParameters["sqlAdministratorLoginPassword"] = $SqlAdminCredential.Password

$StorageResourceGroupName = 'Automation_Portal_Staging'
if ($null -eq (Get-AzResourceGroup -Name $StorageResourceGroupName -Verbose -ErrorAction SilentlyContinue)) {
    New-AzResourceGroup -Name $StorageResourceGroupName -Location $Location -Verbose -Force -ErrorAction Stop
}

$StorageAccountName = 'portal' + ((Get-AzContext).Subscription.Id).Replace('-', '').substring(0, 18)
$StorageAccount = (Get-AzStorageAccount | Where-Object { $_.StorageAccountName -eq $StorageAccountName })
if($null -eq $StorageAccount) {
    $StorageAccount = New-AzStorageAccount -StorageAccountName $StorageAccountName -Type 'Standard_LRS' -ResourceGroupName $StorageResourceGroupName -Location "$Location" -AllowBlobPublicAccess $true
}

$StorageContainerName = $ResourceGroupName.ToLowerInvariant() + '-portalartifacts'
if ($StorageContainerName.length -gt 63) {
    $StorageContainerName = $StorageContainerName.Substring(0, 63)
}

$ArtifactStagingLocation = $StorageAccount.Context.BlobEndPoint + $StorageContainerName + "/"   
$OptionalParameters[$ArtifactsLocationName] = $ArtifactStagingLocation   

New-AzStorageContainer -Name $StorageContainerName -Context $StorageAccount.Context -ErrorAction SilentlyContinue *>&1

$ArtifactStagingDirectory = [System.IO.Path]::GetFullPath([System.IO.Path]::Combine($PSScriptRoot, "ArmTemplate"))
$ArtifactFilePaths = Get-ChildItem $ArtifactStagingDirectory -Recurse -File | ForEach-Object -Process { $_.FullName }
foreach ($SourcePath in $ArtifactFilePaths) {
    $blobName = ($SourcePath -ireplace [regex]::Escape($ArtifactStagingDirectory), "").TrimStart("/").TrimStart("\")
    Set-AzStorageBlobContent -File $SourcePath -Blob $blobName -Container $StorageContainerName -Context $StorageAccount.Context -Force
}

$sasToken = (New-AzStorageContainerSASToken -Container $StorageContainerName -Context $StorageAccount.Context -Permission r -ExpiryTime (Get-Date).AddHours(4))
$OptionalParameters[$ArtifactsLocationSasTokenName] = ConvertTo-SecureString $sasToken -AsPlainText -Force
$TemplateArgs.Add('TemplateUri', $ArtifactStagingLocation + "mainTemplate.json"+ $sasToken)

if ($null -eq (Get-AzResourceGroup -Name $ResourceGroupName -Location $Location -Verbose -ErrorAction SilentlyContinue)) {
    New-AzResourceGroup -Name $ResourceGroupName -Location $Location -Verbose -Force -ErrorAction Stop
}

$DeploymentName = 'mainTemplate' + '-' + ((Get-Date).ToUniversalTime()).ToString('MMdd-HHmm')

$ErrorActionPreference = 'Continue' 
$ResourceGroupDeployment = New-AzResourceGroupDeployment -Name $DeploymentName `
    -ResourceGroupName $ResourceGroupName `
    @TemplateArgs `
    @OptionalParameters `
    -Verbose `
    -ErrorVariable ErrorMessages
$ErrorActionPreference = 'Stop' 

if ($ErrorMessages) {
    Write-Output '', 'Template deployment returned the following errors:', '', @(@($ErrorMessages) | ForEach-Object { $_.Exception.Message })
    Write-Error "Deployment failed."
    Exit 1
}

#####################################################################
# Remove Staging Resource Group
#####################################################################
Write-Progress -Activity "Deploying App Service" -Id 2 -Status "Removing Staging Resource Group" -PercentComplete 64
Remove-AzResourceGroup -Name $StorageResourceGroupName -Force

#####################################################################
# Output
#####################################################################
Write-Progress -Activity "Deploying App Service" -Id 2 -Status "Preparing output" -PercentComplete 80
Write-Output @{
    AppHostName = $ResourceGroupDeployment.Outputs["appHostName"].Value
    AppServiceName = $ResourceGroupDeployment.Outputs["appServiceName"].Value
    AppRegistrationName = $ResourceGroupDeployment.Outputs["appRegistrationName"].Value
    AppServicePlanName = $ResourceGroupDeployment.Outputs["appServicePlanName"].Value
}

Write-Progress -Activity "Deploying App Service" -Id 2 -Status "Complete" -PercentComplete 100
 
Exit 0