#Requires -Version 5.1
#Requires -Modules @{ ModuleName="Microsoft.Graph.Applications"; RequiredVersion="2.5.0" }
#Requires -Modules @{ ModuleName="Microsoft.Graph.Identity.SignIns"; RequiredVersion="2.5.0" }

Param(
    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string] 
    $TenantId,

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string] 
    $AppRegistrationName,

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $AppHostName,

    [Parameter(Mandatory = $false)] 
    [ValidateNotNull()]
    [ValidateRange(1, 24)]
    [int] 
    $ClientSecretExpiresInMonths = 12
)

<######################################################################
    Grant application consent.
######################################################################>
function Grant-Consent {
    param (
        $RequiredResourcesAccess, 
        $ServicePrincipal
    )

    Try {
        ForEach ($resourceAppAccess in $requiredResourcesAccess) {
            $resourceApp = Get-MgServicePrincipal -Filter "appId eq '$($resourceAppAccess.ResourceAppId)'"
            ForEach ($permission in $resourceAppAccess.ResourceAccess | Where-Object { $_.Type -eq "Role" }) {
                $params = @{
                    "PrincipalId" = $ServicePrincipal.Id
                    "ResourceId" = $ResourceApp.Id
                    "AppRoleId" = $permission.Id
                }
                
                New-MgServicePrincipalAppRoleAssignment -ServicePrincipalId $ServicePrincipal.Id -BodyParameter $params
            }
        }

        ForEach ($resourceAppAccess in $requiredResourcesAccess) {
            $delegatedPermissions = @()
            $resourceApp = Get-MgServicePrincipal -Filter "appId eq '$($resourceAppAccess.ResourceAppId)'"
            ForEach ($permission in $resourceAppAccess.ResourceAccess | Where-Object { $_.Type -eq "Scope" }) {
                $permissionObj = $resourceApp.OAuth2PermissionScopes | Where-Object { $_.Id -contains $permission.Id }
                $delegatedPermissions += $permissionObj.Value
            }

            if ($delegatedPermissions) {
                $params = @{
                    "ClientId" = $ServicePrincipal.Id
                    "ConsentType" = "AllPrincipals"
                    "ResourceId" =  $ResourceApp.Id
                    "Scope" = $delegatedPermissions -Join " "
                  }
                  
                  New-MgOauth2PermissionGrant -BodyParameter $params
            }
        }
    }
    Catch {
        Write-Host -f Red "Failed to grant consent."
        Exit
    }
}

<######################################################################
    Add client application.
######################################################################>
function Add-ClientApplication {
    param (
        $App
    )

    Try {
        $Oauth2Permission = $App.Api.OAuth2PermissionScopes | Where-Object { $_.Value -eq 'user_impersonation' }

        $Api = @{
            PreAuthorizedApplications = @(
                @{
                    AppId = $App.AppId
                    DelegatedPermissionIds = $Oauth2Permission.Id
                }
            )
        }

        Update-MgApplication -ApplicationId $App.Id -Api $Api
    }
    Catch {
        Write-Host -f Red "Failed to add client application."
        Exit
    }
}

#####################################################################
# Connect to Microsoft Graph
#####################################################################
# Check graph already connected and is correct for TenantId. If not disconnect.
Write-Progress -Activity "Installing App Registration" -Id 3 -Status "Checking Microsoft Graph context" -PercentComplete 10
$GraphContext = Get-MgContext
if($GraphContext -and $GraphContext.TenantId -ne $TenantId) {
   Disconnect-MgGraph
}

Connect-MgGraph -TenantId $TenantId -Scopes "Application.Read.All", "Application.ReadWrite.All", "User.Read.All", "DelegatedPermissionGrant.ReadWrite.All", "AppRoleAssignment.ReadWrite.All"

#####################################################################
# Check if application registration already exists
#####################################################################
Write-Progress -Activity "Installing App Registration" -Id 3 -Status "Checking application name" -PercentComplete 20
if (Get-MgApplication -Filter "DisplayName eq '$($AppRegistrationName)'" -ErrorAction SilentlyContinue) {
    Write-Host -f Red "Azure AD Application $AppRegistrationName already exists. The Web Application has not been registered with Azure AD."
    Exit 1
}

#####################################################################
# Create Application
#####################################################################
# Manage -> API permissions -> Configured permissions
$RequiredResourceAcesss = @(
    @{
        ResourceAppId = "00000003-0000-0000-c000-000000000000"
        ResourceAccess = @(
            @{
                Id = "7ab1d382-f21e-4acd-a863-ba3e13f7da61" #Directory.Read.All Application
                Type = "Role"
            },
            @{
                Id = "06da0dbc-49e2-44d2-8312-53f166ab848a" #Directory.Read.All Delegated
                Type = "Scope"
            },
            @{
                Id = "e1fe6dd8-ba31-4d61-89e7-88639da4683d" #User.Read Delegated
                Type = "Scope"
            },
            @{
                Id = "14dad69e-099b-42c9-810b-d002981feec1" #profile Delegated
                Type = "Scope"
            },
            @{
                Id = "37f7f235-527c-4136-accd-4a02d197296e" #openid Delegated
                Type = "Scope"
            }
        )
    }
)

# Manage -> App roles -> App roles
$AppRoles = @( 
    @{
        AllowedMemberTypes = @("User", "Application")
        DisplayName = "administrator"
        Id = New-Guid
        IsEnabled = $true
        Description = "Kelverion Automation Portal administrator role."
        Value = "administrator"
    },
    @{
        AllowedMemberTypes = @("User")
        DisplayName = "operator"
        Id = New-Guid
        IsEnabled = $true
        Description = "Kelverion Automation Portal operator role."
        Value = "operator"
    },
    @{
        AllowedMemberTypes = @("User")
        DisplayName = "dashboard operator"
        Id = New-Guid
        IsEnabled = $true
        Description = "Kelverion Automation Portal dashboard operator role."
        Value = "dashboard"
    },
    @{
        AllowedMemberTypes = @("User")
        DisplayName = "developer"
        Id = New-Guid
        IsEnabled = $true
        Description = "Kelverion Automation Portal developer role."
        Value = "developer"
    }
)

# Manage -> Autentication -> Redirect URIs
$PublicClient = @{ RedirectUris = @("https://login.microsoftonline.com/common/oauth2/nativeclient", "msauth.com.kelverion.AutomationPortal://auth") } 

# Manage -> Authentication -> Implicit grant and hybrid flows
$Web = @{ ImplicitGrantSettings = @{
    EnableAccessTokenIssuance = $true
    EnableIdTokenIssuance = $true
}} 

# Manage -Expose an API
$Api = @{
    oauth2PermissionScopes = @(
        @{
            AdminConsentDescription = "Allow the application to access $AppRegistrationName on behalf of the signed-in user."
            AdminConsentDisplayName = "Access $AppRegistrationName"
            UserConsentDescription  = "Allow the application to access $AppRegistrationName on your behalf."
            UserConsentDisplayName  = "Access $AppRegistrationName"
            Id                      = New-Guid
            IsEnabled               = $true
            Type                    = "User"
            Value                   = "user_impersonation"
        }
    )
}

Write-Progress -Activity "Installing App Registration" -Id 3 -Status "Creating app registration" -PercentComplete 30
$App = New-MgApplication -DisplayName $AppRegistrationName `
    -AppRoles $AppRoles `
    -GroupMembershipClaims "SecurityGroup" `
    -SignInAudience "AzureADMyOrg" `
    -Web $Web `
    -Api $Api `
    -PublicClient $PublicClient

$AdminRole = $App.AppRoles.where({ $_.DisplayName -eq "administrator"})
$AdministatorPermission = @{
    ResourceAppId = $App.AppId
    ResourceAccess = @(
        @{
            Id = $AdminRole.Id
            Type = "Role"
        }
    )
}
$RequiredResourceAcesss += $AdministatorPermission

Update-MgApplication -ApplicationId $App.Id -RequiredResourceAccess $RequiredResourceAcesss

# Assign owner - This is a fix for New-MgApplication not assigning user as documented
$User= Invoke-MgGraphRequest -Method GET https://graph.microsoft.com/v1.0/me
$ObjectId = $User.ID
$NewOwner = @{
     "@odata.id"= "https://graph.microsoft.com/v1.0/directoryObjects/{$ObjectId}"
     }
New-MgApplicationOwnerByRef -ApplicationId $App.Id -BodyParameter $NewOwner

#####################################################################
# Manage -> Certificates & secrets 
#####################################################################
Write-Progress -Activity "Installing App Registration" -Id 3 -Status "Creating client secret" -PercentComplete 40
$PasswordCred = @{
    "displayName" = "AppAccessKey"
    "endDateTime" = (Get-Date).AddMonths($ClientSecretExpiresInMonths)
}
$ClientSecret = Add-MgApplicationPassword -ApplicationId $App.Id -PasswordCredential $PasswordCred

#####################################################################
# Azure Active Directory -> Manage -> Enterprise application
#####################################################################
Write-Progress -Activity "Installing App Registration" -Id 3 -Status "Creating enterprise application" -PercentComplete 50
$ServicePrincipalID= @{ 
    "AppId" = $App.AppId
    "Tags" = @("WindowsAzureActiveDirectoryIntegratedApp")
}
$ServicePrincipal = New-MgServicePrincipal -BodyParameter $ServicePrincipalID

#####################################################################
# Azure Active Directory -> Manage -> Enterprise application -> Manage -> Users and groups -> Add user/group
#####################################################################
Write-Progress -Activity "Installing App Registration" -Id 3 -Status "Adding owner" -PercentComplete 60
$AppRole = $ServicePrincipal.AppRoles | Where-Object { $_.DisplayName -eq "Administrator" }
$Owner = Get-MgApplicationOwner -ApplicationId $App.Id
New-MgUserAppRoleAssignment -UserId $Owner.Id -PrincipalId $Owner.Id -ResourceId $ServicePrincipal.Id -AppRoleID $AppRole.Id

#####################################################################
# Manage -> API permissions -> Configured permissions -> Grant admin consent for
#####################################################################
Write-Progress -Activity "Installing App Registration" -Id 3 -Status "Granting permissions" -PercentComplete 70
Grant-Consent -RequiredResourcesAccess $RequiredResourceAcesss -ServicePrincipal $ServicePrincipal

#####################################################################
# Manage -> Expose an Api -> Add client applications
#####################################################################
Write-Progress -Activity "Installing App Registration" -Id 3 -Status "Adding client application" -PercentComplete 80
Add-ClientApplication -App $App

#####################################################################
# Manage -> Authentication -> Redirect URIs
# Manage -> Branding & properties -> Home page URL
# Manage -> Expose an API -> Application ID URI
#####################################################################
Write-Progress -Activity "Installing App Registration" -Id 3 -Status "Adding URLS" -PercentComplete 90
$Web = @{ 
    RedirectUris = @("https://$AppHostName/api/oauth2-redirect.html", "https://$AppHostName/api/.auth/login/aad/callback", "https://$AppHostName/")
    HomePageUrl = "https://$($AppHostName)"
    LogoutUrl = "https://$($AppHostName)/Account/EndSession"
}

Update-MgApplication -ApplicationId $App.Id -Web $Web -IdentifierUris @("api://$($App.AppId)") -IsFallbackPublicClient

#####################################################################
# Disonnect from Microsoft Graph
#####################################################################
Disconnect-MgGraph

#####################################################################
# Output
#####################################################################
Write-Progress -Activity "Installing App Registration" -Id 3 -Status "Preparing output" -PercentComplete 95
Write-Output @{
    ApplicationId = $($App.AppId)
    ClientSecret = $ClientSecret
    ClientSecretEndDate = $ClientSecret.EndDateTime
}

Write-Progress -Activity "Installing App Registration" -Id 3 -Status "Complete" -PercentComplete 100

Write-Host -ForegroundColor Yellow "******************************************************************************"
Write-Host -ForegroundColor Yellow "Client Secret: $($ClientSecret.SecretText)"
Write-Host -ForegroundColor Yellow "Automation Portal Client Secret will expire on: $($ClientSecret.EndDateTime.ToString())"
Write-Host -ForegroundColor Yellow "******************************************************************************"

Exit 0