#Requires -Version 5.1
#Requires -Modules @{ ModuleName="Az.Accounts"; RequiredVersion="2.13.0" }
#Requires -Modules @{ ModuleName="Az.Resources"; RequiredVersion="6.10.0" }
#Requires -Modules @{ ModuleName="Az.Storage"; RequiredVersion="5.10.0" }
#Requires -Modules @{ ModuleName="Az.KeyVault"; RequiredVersion="4.11.0" }
#Requires -Modules @{ ModuleName="Az.Sql"; RequiredVersion="4.10.0" }
#Requires -Modules @{ ModuleName="SqlServer"; RequiredVersion="22.1.1" }

Param(
    [ValidateNotNull()]
    [string]
    $TenantId,

    [ValidateNotNull()]
    [string]
    $SubscriptionId,

    [Parameter(Mandatory = $false)] 
    [ValidateNotNull()]
    [string]
    $ResourceGroupName = 'KelverionAutomationPortal',

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $AppServiceName,

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $AppServicePlanName,
   
    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $ConnectionString,

    [ValidateNotNull()]
    [string]
    $Location,

    [Parameter(Mandatory = $false)]
    [ValidateNotNull()]
    [bool] 
    $UseCurrentAzContext = $false,

    [Parameter(Mandatory = $false)] 
    [ValidateSet("AzureCloud", "AzureUsGovernment", "AzureGermanCloud", "AzureChinaCloud")]
    [string]
    $Environment = "AzureCloud"
)

function SetSqlServerFirewallRules {
    param (
        $ResourceGroupName,
        [string] $ConnectionString
    )

    Try
    { 
        $response = Invoke-WebRequest ifconfig.me/ip
        $ip = $response.Content.Trim()
    
        if($ip -notmatch "\S")
        {
            $ip = Read-Host -Prompt 'Unable to determine the public IP address. This is used when creating an Azure SQL firewall rule. Please specfiy your public IP address.'
        }
    }
    Catch
    {
        $ip = Read-Host -Prompt 'Unable to determine the public IP address. This is used when creating an Azure SQL firewall rule. Please specfiy your public IP address.'
    }
    
    $ConnectionString -match 'Data Source=tcp:(.*).database.windows.net,.*'

    if(!(Get-AzSqlServerFirewallRule -ServerName $Matches[1] -ResourceGroupName $ResourceGroupName -FirewallRuleName "PortalInstallerIp" -ErrorAction SilentlyContinue))
    {
        Write-Progress -Activity "Protecting SQL Database" -Id 5 -Status "Adding firewall rule" -PercentComplete 56
        New-AzSqlServerFirewallRule -ServerName $Matches[1] -ResourceGroupName $ResourceGroupName -FirewallRuleName "PortalInstallerIp" -StartIpAddress $ip -EndIpAddress $ip
    }
    else
    {
        Write-Progress -Activity "Protecting SQL Database" -Id 5 -Status "Updating firewall rule" -PercentComplete 70
        Set-AzSqlServerFirewallRule -ServerName $Matches[1] -ResourceGroupName $ResourceGroupName -FirewallRuleName "PortalInstallerIp" -StartIpAddress $ip -EndIpAddress $ip
    }
}
 
#####################################################################
# Connect to Azure
#####################################################################
Write-Progress -Activity "Protecting SQL Database" -Id 5 -Status "Checking Azure context" -PercentComplete 14
$AzContext = Get-AzContext
if($UseCurrentAzContext -eq $false -or $null -eq $AzContext -or $AzContext.Tenant.Id -ne $TenantId -or $AzContext.Subscription.Id -ne $SubscriptionId) {
    Connect-AzAccount -TenantId $TenantId -SubscriptionId $SubscriptionId -Environment $Environment 
}

#####################################################################
# Validate Resource Group
#####################################################################
$ResourceGroup = Get-AzResourceGroup -Name $ResourceGroupName -Verbose -ErrorAction SilentlyContinue
if ($null-eq $ResourceGroup) {
    Write-Host -f Red "The Resource Group: $ResourceGroupName does not exist."
    Exit 1
}

#####################################################################
# Database firewall rule
#####################################################################
SetSqlServerFirewallRules -ResourceGroupName $ResourceGroupName -ConnectionString $ConnectionString

#####################################################################
# Cert
#####################################################################
$Path = (Get-Location).Path
$CertFile = $Path + "\Automation Portal.pfx"
$PasswordSecure = Read-Host -assecurestring "Please enter a password for the Always Encrypted Certificate."
$PasswordInPlainText = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($PasswordSecure))
$Cert = New-SelfSignedCertificate -Subject "AutomationPortalAlwaysEncryptedCert" -CertStoreLocation Cert:CurrentUser\My -KeyExportPolicy Exportable -Type DocumentEncryptionCert -KeyUsage DataEncipherment -KeySpec KeyExchange
$Cert | Export-PfxCertificate -FilePath $CertFile -Password $PasswordSecure

$AppServicePlan = Get-AzResource -Name $AppServicePlanName `
    -ResourceGroupName $ResourceGroupName `
    -ResourceType "Microsoft.Web/serverfarms"
 
$AppServicePlanId = $AppServicePlan.ResourceId

if($AppServicePlan.Sku.Name -eq "F1" -or $AppServicePlan.Sku.Name -eq "D1")
{
    Set-AzAppServicePlan -ResourceGroupName $ResourceGroupName -Name $AppServicePlanName -Tier Basic
}
 
$CertName = "AutomationPortalAlwaysEncryptedCert"
 
$PfxBlob = [System.Convert]::ToBase64String([System.IO.File]::ReadAllBytes($CertFile))
$PropertiesObject = @{
    pfxBlob      = $PfxBlob
    password     = $PasswordInPlainText
    serverFarmId = $AppServicePlanId
}
 
New-AzResource -Name $CertName `
    -Location $Location `
    -PropertyObject $PropertiesObject `
    -ResourceGroupName $ResourceGroupName `
    -ResourceType Microsoft.Web/certificates `
    -Force

#####################################################################
# Configuration
#####################################################################
Write-Progress -Activity "Protecting SQL Database" -Id 5 -Status "Configuring app service" -PercentComplete 75
$App = Get-AzWebApp -Name $AppServiceName

$AppSettings = @{}
$AppSettings.add("WEBSITE_LOAD_CERTIFICATES", $Cert.Thumbprint)

foreach ($setting in $app.SiteConfig.AppSettings)
{
    $AppSettings.add($setting.Name, $setting.Value)
}

Write-Progress -Activity "Protecting SQL Database" -Id 5 -Status "Updating app service" -PercentComplete 80
Set-AzWebApp -AppSettings $AppSettings -Name $AppServiceName -ResourceGroupName $ResourceGroupName

#####################################################################
# Database
#####################################################################
Write-Progress -Activity "Protecting SQL Database" -Id 5 -Status "Encryptiong Database columns" -PercentComplete 84
$Database = Get-SqlDatabase -ConnectionString $connectionString

# Create a SqlColumnMasterKeySettings object for your column master key. 
$CmkSettings = New-SqlCertificateStoreColumnMasterKeySettings -CertificateStoreLocation "CurrentUser" -Thumbprint $Cert.Thumbprint

# Create column master key metadata object (referencing your certificate), named CMK1, in the database.
$CmkName = "CMK_KA1"
New-SqlColumnMasterKey -Name $CmkName -InputObject $Database -ColumnMasterKeySettings $CmkSettings

$CekName = "CEK_KA1"
New-SqlColumnEncryptionKey -Name $CekName -InputObject $Database -ColumnMasterKey $CmkName

# Get encryption info
$EncryptionQuery = "SELECT t.name AS TableName, c.name AS ColumnName, k.name AS KeyName, c.encryption_type_desc, c.encryption_algorithm_name FROM sys.columns c INNER JOIN sys.column_encryption_keys k ON c.column_encryption_key_id = k.column_encryption_key_id INNER JOIN sys.tables t ON c.object_id = t.object_id WHERE encryption_type IS NOT NULL and t.name = 'Query'"
$EncryptionMeta = Invoke-Sqlcmd -ConnectionString $ConnectionString -Query $EncryptionQuery

# Encrypt the SSN and Salary columns 
$Ces = @()
$Ces += New-SqlColumnEncryptionSettings -ColumnName "dbo.SecureRequestData.Value" -EncryptionType "Deterministic" -EncryptionKey $cekName

if($null -eq $EncryptionMeta)
{
    $Ces += New-SqlColumnEncryptionSettings -ColumnName "dbo.Query.Password" -EncryptionType "Deterministic" -EncryptionKey $cekName
}

Set-SqlColumnEncryption -InputObject $Database -ColumnEncryptionSettings $Ces -LogFileDirectory .

Write-Progress -Activity "Protecting SQL Database" -Id 5 -Status "Complete" -PercentComplete 100

Write-Host -ForegroundColor Yellow "******************************************************************************"
Write-Host -ForegroundColor Yellow "Certificate: $CertFile"
Write-Host -ForegroundColor Yellow "Column Master Key: $CmkName"
Write-Host -ForegroundColor Yellow "Column Encryption Key $CekName"
Write-Host -ForegroundColor Yellow "******************************************************************************"
