#Requires -Version 5.1
#Requires -Modules @{ ModuleName="Az.Accounts"; RequiredVersion="2.13.0" }
#Requires -Modules @{ ModuleName="Az.Resources"; RequiredVersion="6.10.0" }
#Requires -Modules @{ ModuleName="Az.Storage"; RequiredVersion="5.10.0" }
#Requires -Modules @{ ModuleName="Az.KeyVault"; RequiredVersion="4.11.0" }
#Requires -Modules @{ ModuleName="Az.Websites"; RequiredVersion="3.1.1" }
#Requires -Modules @{ ModuleName="Az.Sql"; RequiredVersion="4.10.0" }
#Requires -Modules @{ ModuleName="Microsoft.Graph.Applications"; RequiredVersion="2.5.0" }
#Requires -Modules @{ ModuleName="Microsoft.Graph.Identity.SignIns"; RequiredVersion="2.5.0" }
#Requires -Modules @{ ModuleName="SqlServer"; RequiredVersion="22.1.1" }

Param(
    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $TenantId,

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $SubscriptionId,

    [Parameter(Mandatory = $false)] 
    [ValidateNotNull()]
    [string]
    $ResourceGroupName = 'KelverionAutomationPortal',

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $Location,

    [Parameter(ParameterSetName = 'AppRegistrationTenantId')] 
    [ValidateNotNull()]
    [string]
    $AppRegistrationTenantId,

    [Parameter(Mandatory = $false)] 
    [ValidateNotNull()]
    [System.Management.Automation.Credential()]
    [System.Management.Automation.PSCredential] 
    $SqlAdminCredential = [System.Management.Automation.PSCredential]::Empty,

    [Parameter(Mandatory = $false)] 
    [ValidateNotNull()]
    [ValidateRange(1, 24)]
    [int] 
    $ClientSecretExpiresInMonths = 12,

    [Parameter(Mandatory = $false)] 
    [ValidateSet("AzureCloud", "AzureUsGovernment", "AzureGermanCloud", "AzureChinaCloud")]
    [string]
    $Environment = "AzureCloud"
)

#####################################################################
# Connect to Azure
#####################################################################
Connect-AzAccount -TenantId $TenantId -SubscriptionId $SubscriptionId -Environment $Environment 

##############################################################################
# Step 1: Deploy App Service
##############################################################################
Write-Progress -Activity "Installing" -Id 1 -Status "Step 1 of 4: Deploying App Service" -PercentComplete 25
$Deployment = .\Deploy-AutomationPortalAppService.ps1 `
    -TenantId $TenantId `
    -SubscriptionId $SubscriptionId `
    -ResourceGroupName $ResourceGroupName `
    -SqlAdminCredential $SqlAdminCredential `
    -Location $Location `
    -UseCurrentAzContext $true `
    -Environment $Environment
if($LastExitCode -ne 0) {
    Exit 1
}

##############################################################################
# Step 2: Install App Registration
##############################################################################
Write-Progress -Activity "Installing"  -Id 1 -Status "Step 2 of 4: Registering App" -PercentComplete 50

if($AppRegistrationTenantId) {
    $AppRegistration = .\Install-AutomationPortalAppRegistration.ps1 `
    -TenantId $AppRegistrationTenantId `
    -AppRegistrationName $Deployment.AppRegistrationName `
    -AppHostName $Deployment.AppHostName `
    -ClientSecretExpiresInMonths $ClientSecretExpiresInMonths
}
else {
    $AppRegistration = .\Install-AutomationPortalAppRegistration.ps1 `
    -TenantId $TenantId `
    -AppRegistrationName $Deployment.AppRegistrationName `
    -AppHostName $Deployment.AppHostName `
    -ClientSecretExpiresInMonths $ClientSecretExpiresInMonths
}

if($LastExitCode -ne 0) {
    Exit 1
}

##############################################################################
# Step 3: Initialize App Service
##############################################################################
Write-Progress -Activity "Installing"  -Id 1 -Status "Step 3 of 4: Initializing App Service" -PercentComplete 75

if($AppRegistrationTenantId) {
    $AppSettings = .\Initialize-AutomationPortalAppService.ps1 `
    -TenantId $TenantId `
    -SubscriptionId $SubscriptionId `
    -AppServiceName $Deployment.AppServiceName `
    -ApplicationId $AppRegistration.ApplicationId `
    -AppRegistrationTenantId $AppRegistrationTenantId `
    -ClientSecret $AppRegistration.ClientSecret.SecretText `
    -UseCurrentAzContext $true `
    -Environment $Environment
}
else {
    $AppSettings = .\Initialize-AutomationPortalAppService.ps1 `
    -TenantId $TenantId `
    -SubscriptionId $SubscriptionId `
    -AppServiceName $Deployment.AppServiceName `
    -ApplicationId $AppRegistration.ApplicationId `
    -ClientSecret $AppRegistration.ClientSecret.SecretText `
    -UseCurrentAzContext $true `
    -Environment $Environment
}

##############################################################################
# Step 4: Protect SQL Database
##############################################################################
Write-Progress -Activity "Installing"  -Id 1 -Status "Step 4 of 4: Protecting SQL Database" -PercentComplete 100

.\Protect-AutomationPortalSqlDatabase `
    -TenantId $TenantId `
    -SubscriptionId $SubscriptionId `
    -ResourceGroupName $ResourceGroupName `
    -Location $Location `
    -UseCurrentAzContext $true `
    -ConnectionString $AppSettings.ConnectionString `
    -AppServiceName $Deployment.AppServiceName `
    -AppServicePlanName $Deployment.AppServicePlanName

Write-Host -ForegroundColor Yellow "******************************************************************************"
Write-Host -ForegroundColor Yellow "App Host Name: https://$($Deployment.AppHostName)"
Write-Host -ForegroundColor Yellow "App ID: $($AppRegistration.ApplicationId)"
Write-Host -ForegroundColor Yellow "App Service Name: $($Deployment.AppServiceName)"
Write-Host -ForegroundColor Yellow "AAD App Registration Name: $($Deployment.AppRegistrationName)"
Write-Host -ForegroundColor Yellow "Automation Portal Client Secret will expire on: $($AppRegistration.ClientSecretEndDate.ToString())"
Write-Host -ForegroundColor Yellow "******************************************************************************"

Write-Host -ForegroundColor Green "******************************************************************************"
Write-Host -ForegroundColor Green "Installation Complete"
Write-Host -ForegroundColor Green "******************************************************************************"