﻿#Requires -Version 5.1
#Requires -Modules @{ ModuleName="Az.Accounts"; RequiredVersion="2.13.0" }
#Requires -Modules @{ ModuleName="Az.Resources"; RequiredVersion="6.10.0" }
#Requires -Modules @{ ModuleName="Az.Websites"; RequiredVersion="3.1.1" }
#Requires -Modules @{ ModuleName="Az.Sql"; RequiredVersion="4.10.0" }
#Requires -Modules @{ ModuleName="SqlServer"; RequiredVersion="22.1.1" }
#Requires -Modules @{ ModuleName="Microsoft.Graph.Applications"; RequiredVersion="2.5.0" }
#Requires -Modules @{ ModuleName="Microsoft.Graph.Identity.SignIns"; RequiredVersion="2.5.0" }

Param(
    [Parameter(Mandatory=$true)] 
    [ValidateNotNull()]
    [string]
    $TenantId,

    [Parameter(Mandatory=$true)] 
    [ValidateNotNull()]
    [string]
    $SubscriptionId,

    [Parameter(Mandatory = $false)] 
    [string] 
    $AppRegistrationTenantId,
    
    [Parameter(Mandatory=$true)]
    [string] $AppServiceName,

    [Parameter(Mandatory = $false)] 
    [ValidateSet("AzureCloud", "AzureUsGovernment", "AzureGermanCloud", "AzureChinaCloud")]
    [string]
    $Environment = "AzureCloud"
)

function SetSqlServerFirewallRules {
    param (
        $AppService,
        [string] $ConnectionString
    )

    Try
    { 
        $response = Invoke-WebRequest ifconfig.me/ip
        $ip = $response.Content.Trim()
    
        if($ip -notmatch "\S")
        {
            $ip = Read-Host -Prompt 'Unable to determine the public IP address. This is used when creating an Azure SQL firewall rule. Please specfiy your public IP address.'
        }
    }
    Catch
    {
        $ip = Read-Host -Prompt 'Unable to determine the public IP address. This is used when creating an Azure SQL firewall rule. Please specfiy your public IP address.'
    }
    
    $ConnectionString -match 'Data Source=tcp:(.*).database.windows.net,.*'
    
    if(!(Get-AzSqlServerFirewallRule -ServerName $Matches[1] -ResourceGroupName $AppService.ResourceGroup -FirewallRuleName "PortalInstallerIp" -ErrorAction SilentlyContinue))
    {
        New-AzSqlServerFirewallRule -ServerName $Matches[1] -ResourceGroupName $AppService.ResourceGroup -FirewallRuleName "PortalInstallerIp" -StartIpAddress $ip -EndIpAddress $ip
    }
    else
    {
        Set-AzSqlServerFirewallRule -ServerName $Matches[1] -ResourceGroupName $AppService.ResourceGroup -FirewallRuleName "PortalInstallerIp" -StartIpAddress $ip -EndIpAddress $ip
    }
}

function Add-ReplyUrls {
    param (
        $App
    )

    Try {
        $PublicClient = @{ RedirectUris = @("https://login.microsoftonline.com/common/oauth2/nativeclient", "msauth.com.kelverion.AutomationPortal://auth") } 
        Update-MgApplication -ApplicationId $App.Id -PublicClient $PublicClient
    }
    Catch {
        Write-Host -f Red "Failed to add reply Urls."
        Exit
    }
}

function UpdateAppRegistration {
    param (
        $AdApplicationName
    )

    $App = Get-MgApplication -Filter "DisplayName eq '$AdApplicationName'"

    foreach($role in $App.AppRoles)
    {
        $role.IsEnabled = $false
    }
    
    Update-MgApplication -ApplicationId $App.Id -AppRoles $App.AppRoles
    
    foreach($role in $App.AppRoles)
    {
        if($role.DisplayName -eq "administrator") 
        {
            $role.AllowedMemberTypes += "Application"
        }
    
        $role.IsEnabled = $true
    }
    
    Update-MgApplication -ApplicationId $App.Id -AppRoles $App.AppRoles
    
    $App = Get-MgApplication -Filter "DisplayName eq '$AdApplicationName'"
    
    $AdminRole = $App.AppRoles.where({ $_.DisplayName -eq "administrator"})
    $AdministatorPermission = @{
        ResourceAppId = $App.AppId
        ResourceAccess = @(
            @{
                Id = $AdminRole.Id
                Type = "Role"
            }
        )
    }
    
    $App.RequiredResourceAccess += $AdministatorPermission
    
    Update-MgApplication -ApplicationId $App.Id -RequiredResourceAccess $App.RequiredResourceAccess
    
    $ServicePrincipal = Get-MgServicePrincipal -Filter "DisplayName eq '$AdApplicationName'"
    $ResourceApp = Get-MgServicePrincipal -Filter "appId eq '$($App.AppId)'"
    $AdminRole = $App.AppRoles.where({ $_.DisplayName -eq "administrator"})
    
    $params = @{
        "PrincipalId" = $ServicePrincipal.Id
        "ResourceId" = $ResourceApp.Id
        "AppRoleId" = $AdminRole.Id
    }
    
    New-MgServicePrincipalAppRoleAssignment -ServicePrincipalId $ServicePrincipal.Id -BodyParameter $params | 
      Format-List Id, AppRoleId, CreatedDateTime, PrincipalDisplayName, PrincipalId, PrincipalType, ResourceDisplayName
}

$version = "3.8"

$supported36UpgradeVersions = @("3.6.8174.28938")
$support37UpgradeVersions = @("3.7.8424.19002", "3.7.8489.19139")

$supportedUpgradeVersions = @("3.7.8424.19002", "3.7.8489.19139")
$supportedUpgradeVersions += $supported36UpgradeVersions
$supportedUpgradeVersions += $support37UpgradeVersions

# Connect to Azure
Connect-AzAccount -TenantId $TenantId -SubscriptionId $SubscriptionId -Environment $Environment -ErrorAction Stop 

# Connect to Azure Active Directory
if ([string]::IsNullOrEmpty($AppRegistrationTenantId)) {
    Connect-MgGraph -TenantId $TenantId -Scopes "Application.Read.All", "Application.ReadWrite.All", "User.Read.All", "DelegatedPermissionGrant.ReadWrite.All", "AppRoleAssignment.ReadWrite.All"
}
else {
    Connect-MgGraph -TenantId $AppRegistrationTenantId -Scopes "Application.Read.All", "Application.ReadWrite.All", "User.Read.All", "DelegatedPermissionGrant.ReadWrite.All", "AppRoleAssignment.ReadWrite.All"
}      

# Get App Service
$AppService = Get-AzWebApp -Name $AppServiceName
if($null -eq $AppService) {
    Write-Host -f Yellow "The web app name: $AppServiceName not found."
    Exit 1
}

# Get App Service Plan
$AppServicePlan = Get-AzAppServicePlan -ResourceGroupName $AppService.ResourceGroup
if($null -eq $AppServicePlan) {
    Write-Host -f Yellow "The app service plan was not found."
    Exit 1
}

# Get Azure Active Directory App Registration
$AdApplicationName = "kelverionautomationportal" + $AppServiceName.Replace("app-kelverionautomationportalweb", "")
$App = Get-MgApplication -Filter "DisplayName eq '$($AdApplicationName)'"
if($null -eq $App) {
    $App = Get-MgApplication -Filter "DisplayName eq '$($AppServiceName)'"
    if($null -eq $App) {
        Write-Host -f Yellow "The Azure AD app name: $AppServiceName not found."
        Exit
    }

    $AdApplicationName = $AppServiceName
}

# Get Publishin Profile
$xml = [xml](Get-AzWebAppPublishingProfile -Name $AppServiceName -ResourceGroupName $AppService.ResourceGroup -OutputFile null)
$connectionString = $xml.SelectNodes("//publishProfile[@publishMethod=`"FTP`"]/@SQLServerDBConnectionString").value
$username = $xml.SelectNodes("//publishProfile[@publishMethod=`"FTP`"]/@userName").value
$password = $xml.SelectNodes("//publishProfile[@publishMethod=`"FTP`"]/@userPWD").value
$url = $xml.SelectNodes("//publishProfile[@publishMethod=`"FTP`"]/@publishUrl").value
$credentials = New-Object System.Net.NetworkCredential($username, $password)

# Database firewall rule
SetSqlServerFirewallRules -AppService $AppService  -ConnectionString $connectionString

# Get Portal version
$config = Invoke-Sqlcmd -ConnectionString $connectionString -Query "select ConfigValue from Config where ConfigKey = 'Version'"

$actualVersion = ""
$update = $false
$config | ForEach-Object {
    if ($SupportedUpgradeVersions -contains $_.ConfigValue) {
        $update = $true
        $actualVersion = $_.ConfigValue
    }
}

if($update)
{
    # Stop Automation Portal
    Stop-AzWebApp -ResourceGroupName $AppService.ResourceGroup -Name $AppServiceName

    # Update configuration
    $AppSettings = @{}
    $AppSettings.add("Kelverion:Portal:Version", $version)
    
    foreach ($setting in $AppService.SiteConfig.AppSettings)
    {
        if($setting.Name -ne "Kelverion:Portal:Version")
        {
            $AppSettings.add($setting.Name, $setting.Value)
        }
    }
    
    Set-AzWebApp -AppSettings $AppSettings -Name $AppServiceName -ResourceGroupName $AppService.ResourceGroup

    # Update Reply URLS
    if ($supported36UpgradeVersions -contains $actualVersion) {
        # Manage -> Authentication -> Redirect URIs
        Add-ReplyUrls -App $App
    }

    # Update App Registration
    UpdateAppRegistration -AdApplicationName $AdApplicationName
       
    # Update Database
    Invoke-Sqlcmd -ConnectionString $connectionString -InputFile UpdateTables.sql

    # Protect Database
    .\Protect-AutomationPortalSqlDatabase `
        -TenantId $TenantId `
        -SubscriptionId $SubscriptionId `
        -ResourceGroupName $AppService.ResourceGroup `
        -Location $AppService.Location `
        -UseCurrentAzContext $true `
        -ConnectionString $connectionString `
        -AppServiceName $AppServiceName `
        -AppServicePlanName $AppServicePlan.Name

    # Publish
    Publish-AzWebapp -ResourceGroupName $AppService.ResourceGroup -Name $AppServiceName -ArchivePath "$PSScriptRoot\Kelverion.Automation.Portal.zip" -Force -Timeout 1200000

    # Update Connection String
    if($ConnectionString.ToLower().Contains("column encryption setting") -eq $false) 
    {
        $UpdatedConnectionString = $ConnectionString + "Column Encryption Setting=Enabled;"
        $ConnectionStrings =  @{ PortalConnection = @{ Type ='SQLAzure'; Value = $UpdatedConnectionString } }
        Set-AzWebApp -Name $AppServiceName -ResourceGroupName $AppService.ResourceGroup -ConnectionStrings $ConnectionStrings
    }

    # Always On
    Set-AzWebApp -Name $AppServiceName -ResourceGroupName $AppService.ResourceGroup -AlwaysOn $True

    # Start Automation Portal
    Start-AzWebApp -ResourceGroupName $AppService.ResourceGroup -Name $AppServiceName

    Write-Host -ForegroundColor Green "******************************************************************************"
    Write-Host -ForegroundColor Green "Update Complete"
    Write-Host -ForegroundColor Green "******************************************************************************"
}
else {
    Write-Host -ForegroundColor Yellow "******************************************************************************"
    Write-Host -ForegroundColor Yellow "Update Not Complete"
    Write-Host -ForegroundColor Yellow "Incorrect Portal Version: $($actualVersion)"
    Write-Host -ForegroundColor Yellow "Required Version: $($supportedUpgradeVersions)"
    Write-Host -ForegroundColor Yellow "******************************************************************************"
}

Exit 0