#Requires -Version 5.1
#Requires -Modules @{ ModuleName="Az.Accounts"; RequiredVersion="2.13.0" }
#Requires -Modules @{ ModuleName="Az.Resources"; RequiredVersion="6.10.0" }
#Requires -Modules @{ ModuleName="Az.Storage"; RequiredVersion="5.10.0" }
#Requires -Modules @{ ModuleName="Az.KeyVault"; RequiredVersion="4.11.0" }
#Requires -Modules @{ ModuleName="Az.Sql"; RequiredVersion="4.10.0" }
#Requires -Modules @{ ModuleName="SqlServer"; RequiredVersion="22.1.1" }

Param(
    [ValidateNotNull()]
    [string]
    $TenantId,

    [ValidateNotNull()]
    [string]
    $SubscriptionId,

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $AppServiceName,

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $CurrentColumnMasterKeyName,

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $NewColumnMasterKeyName,

    [Parameter(Mandatory = $true)] 
    [ValidateNotNull()]
    [string]
    $CertificateName,

    [Parameter(Mandatory = $false)] 
    [ValidateSet("AzureCloud", "AzureUsGovernment", "AzureGermanCloud", "AzureChinaCloud")]
    [string]
    $Environment = "AzureCloud"
)

function SetSqlServerFirewallRules {
    param (
        $ResourceGroupName,
        [string] $ConnectionString
    )

    Try
    { 
        $response = Invoke-WebRequest ifconfig.me/ip
        $ip = $response.Content.Trim()
    
        if($ip -notmatch "\S")
        {
            $ip = Read-Host -Prompt 'Unable to determine the public IP address. This is used when creating an Azure SQL firewall rule. Please specfiy your public IP address.'
        }
    }
    Catch
    {
        $ip = Read-Host -Prompt 'Unable to determine the public IP address. This is used when creating an Azure SQL firewall rule. Please specfiy your public IP address.'
    }
    
    $ConnectionString -match 'Data Source=tcp:(.*).database.windows.net,.*'

    if(!(Get-AzSqlServerFirewallRule -ServerName $Matches[1] -ResourceGroupName $ResourceGroupName -FirewallRuleName "PortalInstallerIp" -ErrorAction SilentlyContinue))
    {
        New-AzSqlServerFirewallRule -ServerName $Matches[1] -ResourceGroupName $ResourceGroupName -FirewallRuleName "PortalInstallerIp" -StartIpAddress $ip -EndIpAddress $ip
    }
    else
    {
        Set-AzSqlServerFirewallRule -ServerName $Matches[1] -ResourceGroupName $ResourceGroupName -FirewallRuleName "PortalInstallerIp" -StartIpAddress $ip -EndIpAddress $ip
    }
}
 
#####################################################################
# Connect to Azure
#####################################################################

Try {
    $res = Connect-AzAccount -TenantId $TenantId -SubscriptionId $SubscriptionId -Environment $Environment
    if ($null -eq $res) {
        Exit 1
    }
}
Catch {
    Exit 1
}

#####################################################################
# Get App Service
#####################################################################
$AppService = Get-AzWebApp -Name $AppServiceName
if($null -eq $AppService) {
    Write-Host -f Yellow "The web app name: $AppServiceName not found."
    Exit 1
}

#####################################################################
# Get App Service Plan
#####################################################################
$AppServicePlan = Get-AzAppServicePlan -ResourceGroupName $AppService.ResourceGroup
if($null -eq $AppServicePlan) {
    Write-Host -f Yellow "The app service plan was not found."
    Exit 1
}
$AppServicePlanId = $AppServicePlan.Id

#####################################################################
# Connection String
#####################################################################
$xml = [xml](Get-AzWebAppPublishingProfile -Name $AppServiceName -ResourceGroupName $AppService.ResourceGroup -OutputFile null)
$ConnectionString = $xml.SelectNodes("//publishProfile[@publishMethod=`"FTP`"]/@SQLServerDBConnectionString").value

#####################################################################
# Database firewall rule
#####################################################################
SetSqlServerFirewallRules -ResourceGroupName $AppService.ResourceGroup -ConnectionString $ConnectionString

#####################################################################
# Cert
#####################################################################
$Path = (Get-Location).Path
$CertFile = $Path + "\$CertificateName.pfx"
$PasswordSecure = Read-Host -assecurestring "Please enter a password for the Always Encrypted Certificate."
$PasswordInPlainText = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($PasswordSecure))
$Cert = New-SelfSignedCertificate -Subject $CertificateName -CertStoreLocation Cert:CurrentUser\My -KeyExportPolicy Exportable -Type DocumentEncryptionCert -KeyUsage DataEncipherment -KeySpec KeyExchange
$Cert | Export-PfxCertificate -FilePath $CertFile -Password $PasswordSecure

$PfxBlob = [System.Convert]::ToBase64String([System.IO.File]::ReadAllBytes($CertFile))
$PropertiesObject = @{
    pfxBlob      = $PfxBlob
    password     = $PasswordInPlainText
    serverFarmId = $AppServicePlanId
}
 
New-AzResource -Name $CertificateName `
    -Location $AppService.Location `
    -PropertyObject $PropertiesObject `
    -ResourceGroupName $AppService.ResourceGroup `
    -ResourceType Microsoft.Web/certificates `
    -Force

#####################################################################
# Configuration
#####################################################################
$App = Get-AzWebApp -Name $AppServiceName

$AppSettings = @{}
$AppSettings.add("WEBSITE_LOAD_CERTIFICATES", $Cert.Thumbprint)

foreach ($setting in $app.SiteConfig.AppSettings)
{
    if($setting.Name -ne "WEBSITE_LOAD_CERTIFICATES")
    {
        $AppSettings.add($setting.Name, $setting.Value)
    }
}

Set-AzWebApp -AppSettings $AppSettings -Name $AppServiceName -ResourceGroupName $AppService.ResourceGroup

# #####################################################################
# Database
# #####################################################################
$Database = Get-SqlDatabase -ConnectionString $ConnectionString

# Create a SqlColumnMasterKeySettings object for your column master key. 
$CmkSettings = New-SqlCertificateStoreColumnMasterKeySettings -CertificateStoreLocation "CurrentUser" -Thumbprint $Cert.Thumbprint

# Create column master key metadata object (referencing your certificate), named CMK1, in the database.
New-SqlColumnMasterKey -Name $NewColumnMasterKeyName -InputObject $Database -ColumnMasterKeySettings $CmkSettings

# Initiate the rotation from the current column master key to the new column master key.
Invoke-SqlColumnMasterKeyRotation -SourceColumnMasterKeyName $CurrentColumnMasterKeyName -TargetColumnMasterKeyName $NewColumnMasterKeyName -InputObject $Database

# Complete the rotation of the old column master key.
Complete-SqlColumnMasterKeyRotation -SourceColumnMasterKeyName $CurrentColumnMasterKeyName  -InputObject $Database

# Remove the old column master key metadata.
Remove-SqlColumnMasterKey -Name $CurrentColumnMasterKeyName -InputObject $Database

Write-Host -ForegroundColor Yellow "******************************************************************************"
Write-Host -ForegroundColor Yellow "Certificate: $CertFile"
Write-Host -ForegroundColor Yellow "Column Master Key: $NewColumnMasterKeyName"
Write-Host -ForegroundColor Yellow "******************************************************************************"